@@ -7,6 +7,7 @@ import binascii
 import base64
 import bizerror
 import typing
+import math
 from decimal import Decimal
 
 
@@ -299,3 +300,55 @@ def int2bytes(value: int):
         value = value // 256
     bs.reverse()
     return bytes(bs)
+
+def substrings(value, lengths=None):
+    value = value or ""
+    subs = set()
+    if lengths is None:
+        lengths = list(range(len(value)))
+    elif isinstance(lengths, int):
+        lengths = [lengths]
+    for length in lengths:
+        for start in range(len(value) - length + 1):
+            sub = value[start: start + length]
+            subs.add(sub)
+    return subs
+
+def combinations(values, length):
+    values = list(values)
+    min_length = min([len(x) for x in values])
+    max_length = max([len(x) for x in values])
+    repeat = math.ceil((length - min_length)/min_length)
+    repeat_incr = False
+    if length <= min_length:
+        repeat_incr = True
+    if length % min_length == 2 and length % max_length == 2:
+        repeat_incr = True
+    if length % min_length == 0 and length % max_length == 0:
+        repeat_incr = True
+    if repeat_incr:
+        repeat += 1
+    check_length = length + 2 * max_length - 1
+    good_words = set()
+    words = set()
+    short_words = set(values)
+    for i in range(repeat):
+        new_words = set()
+        for value in values:
+            for word in short_words:
+                new_word = word + value
+                if len(new_word) >= check_length:
+                    good_words.add(new_word)
+                else:
+                    new_words.add(new_word)
+        short_words = new_words
+        if not short_words:
+            break
+    for x in short_words:
+        if len(x) >= length:
+            good_words.add(x)
+    result_words = set()
+    for word in good_words:
+        for subword in substrings(word, length):
+            result_words.add(subword)
+    return result_words
